@@ -20,7 +20,7 @@ func TestInterpConsistencyBuild(t *testing.T) {
 	}
 	dir := filepath.Join("..", "_test", "tmp")
 	if _, err := os.Stat(dir); os.IsNotExist(err) {
-		if err := os.Mkdir(dir, 0700); err != nil {
+		if err := os.Mkdir(dir, 0o700); err != nil {
 			t.Fatal(err)
 		}
 	}
@@ -51,11 +51,13 @@ func TestInterpConsistencyBuild(t *testing.T) {
 			file.Name() == "init1.go" || // expect error
 			file.Name() == "io0.go" || // use random number
 			file.Name() == "issue-1093.go" || // expect error
+			file.Name() == "issue-1276.go" || // expect error
 			file.Name() == "op1.go" || // expect error
 			file.Name() == "op7.go" || // expect error
 			file.Name() == "op9.go" || // expect error
 			file.Name() == "bltn0.go" || // expect error
 			file.Name() == "method16.go" || // private struct field
+			file.Name() == "method39.go" || // expect error
 			file.Name() == "switch8.go" || // expect error
 			file.Name() == "switch9.go" || // expect error
 			file.Name() == "switch13.go" || // expect error
@@ -98,7 +100,16 @@ func TestInterpConsistencyBuild(t *testing.T) {
 			file.Name() == "server1.go" || // syntax parsing
 			file.Name() == "server0.go" || // syntax parsing
 			file.Name() == "server.go" || // syntax parsing
-			file.Name() == "range9.go" { // expect error
+			file.Name() == "range9.go" || // expect error
+			file.Name() == "unsafe6.go" || // needs go.mod to be 1.17
+			file.Name() == "unsafe7.go" || // needs go.mod to be 1.17
+			file.Name() == "type27.go" || // expect error
+			file.Name() == "type28.go" || // expect error
+			file.Name() == "type29.go" || // expect error
+			file.Name() == "type30.go" || // expect error
+			file.Name() == "type31.go" || // expect error
+			file.Name() == "type32.go" || // expect error
+			file.Name() == "type33.go" { // expect error
 			continue
 		}
 
@@ -218,12 +229,12 @@ func TestInterpErrorConsistency(t *testing.T) {
 		},
 		{
 			fileName:       "issue-1093.go",
-			expectedInterp: "9:6: cannot use type string as type int in assignment",
+			expectedInterp: "9:6: cannot use type untyped string as type int in assignment",
 			expectedExec:   `9:4: cannot use "a" + b() (type string) as type int in assignment`,
 		},
 		{
 			fileName:       "op1.go",
-			expectedInterp: "5:2: invalid operation: mismatched types int and float64",
+			expectedInterp: "5:2: invalid operation: mismatched types int and untyped float",
 			expectedExec:   "5:4: constant 1.3 truncated to integer",
 		},
 		{
